#--
#                 - Mellanox Confidential and Proprietary -
# 
# Copyright (C) Sep 2019, Mellanox Technologies Ltd.  ALL RIGHTS RESERVED.
# 
# Except as specifically permitted herein, no portion of the information,
# including but not limited to object code and source code, may be reproduced,
# modified, distributed, republished or otherwise exploited in any form or by
# any means for any purpose without the prior written permission of Mellanox
# Technologies Ltd. Use of software subject to the terms and conditions
# detailed in the file "LICENSE.txt".
#--

from __future__ import print_function
import sys
import os
import subprocess
from collections import namedtuple


class Command(object):
    def __init__(self, cmd_str):
        self.cmd_str = cmd_str

    def execute(self):
        p = subprocess.Popen(self.cmd_str,
                             stdin=subprocess.PIPE,
                             stdout=subprocess.PIPE,
                             stderr=subprocess.PIPE,
                             universal_newlines=False,
                             shell=True)
        stdout, stderr = p.communicate()
        stat = p.wait()
        return stat, stdout, stderr


class I2cError(Exception):
    pass


class I2cMaster(object):

    def __init__(self, mtusb_device_name):
        self.mtusb_device_name = mtusb_device_name

    def read(self, i2c_slave_address, address_width, address, size):
        cmd = "i2c -a {0} -x {1} {2} read {3:#x} {4:#x}".format(address_width, size, self.mtusb_device_name, i2c_slave_address, address)
        #print(cmd)
        rc, out, err = Command(cmd).execute()
       
        if rc == 0:
            out = out.strip()
            result = []
            for index in range(len(out)-2, -1, -2):
                byte_value = int(out[index:index+2], 16)
                result.append(byte_value)
            return result
        else:
            raise I2cError("Failed to read from i2c slave address {0} ({1})".format(i2c_slave_address, err))

    def scan(self, devices):
        result = []
        for device in devices:
            #print(device)
            try:
                devid = self.read(device.i2c_slave_address, device.address_width, device.devid_address, 4)[0]
                #print(devid)
                if devid == device.devid:
                    result.append(device)
                else:
                    raise RuntimeError("i2c slave on i2c address {0} has devid {1}. Expected devid is {2}".format(device.i2c_slave_address,devid, device.devid))
            except I2cError as err:
                pass
        return result



if __name__ == '__main__':

    rc = 0

    mst_devices_directory = '/dev/mst' # Linux TODO for Windows,etc

    mst_devices = [f for f in os.listdir(mst_devices_directory)]

    for mst_device in mst_devices:

        if 'mtusb' in mst_device and 'cable' not in mst_device: # EVB with a linkx chip

            mst_device = os.path.join(mst_devices_directory, mst_device) # TODO fix to mst_deivce

            # Create a list of known i2c slave (Linkx) devices 
            LinkxDevice = namedtuple('LinkxDevice', 'devid i2c_slave_address address_width devid_address')

            ardbeg = LinkxDevice(devid = 0x6e, i2c_slave_address = 0x66, address_width = 2, devid_address = 0x2c>>2)
            ardbeg_mirrored = LinkxDevice(devid = 0x70, i2c_slave_address = 0x67, address_width = 2, devid_address = 0x2c>>2)
            baritone = LinkxDevice(devid = 0x6b, i2c_slave_address = 0x5e, address_width = 2, devid_address = 0x2c>>2)
            baritone_mirrored = LinkxDevice(devid = 0x71, i2c_slave_address = 0x5f, address_width = 2, devid_address = 0x2c>>2)
            menhit = LinkxDevice(devid = 0x6f, i2c_slave_address = 0x10, address_width = 2, devid_address = 0x18014>>2)

            linkx_devices = [ardbeg, ardbeg_mirrored, baritone, baritone_mirrored, menhit]

            # Scan i2c slaves
            i2c_master = I2cMaster(mst_device) 

            i2c_slaves = i2c_master.scan(linkx_devices)

            if len(i2c_slaves) == 1:
                new_mst_device = "{0}_lx{1}".format(mst_device, i2c_slaves[0].devid)
                #print(new_mst_device)
                open(new_mst_device, 'w').close()
            elif len(i2c_slaves) > 1:
                print("i2c master detected more than one i2c slaves devices", file=sys.stderr)
                rc = 1

        elif 'cable' in mst_device:
            cmd = "mlxcables -d {0} --discover".format(mst_device) # TODO temp. Change to "mlxcables"
            # TODO We don't have FW right now --> Need to run the FW GW resonser (make sure the cable device exists)
            #print(cmd)
            rc, out, err = Command(cmd).execute()

            if rc == 0:
                result = eval(out)
                for device_id, num_of_devices in result:
                    if num_of_devices == 1:
                        new_mst_device_name = "{0}_lx{1}".format(mst_device, device_id)
                        new_mst_device = os.path.join(mst_devices_directory, new_mst_device_name)
                        #print(new_mst_device)
                        open(new_mst_device, 'w').close()
                    elif num_of_devices >1:
                        for device_num in range(num_of_devices):
                            new_mst_device_name = "{0}_lx{1}_{2}".format(mst_device, device_id, device_num)
                            new_mst_device = os.path.join(mst_devices_directory, new_mst_device_name)
                            #print(new_mst_device)
                            open(new_mst_device, 'w').close()
                    else:
                        rc = 1 # ERROR!!!
            

    sys.exit(rc)
